# Copyright (c) 2003 Sylvain Thenault (thenault@nerim.net)
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""astng loader, avoid multible astng build of a same module
"""

__author__ = "Sylvain Thenault"
__revision__ = "$Id: manager.py,v 1.12 2003/12/10 08:15:09 syt Exp $"

import sys
import os
from os.path import dirname

from logilab.common.modutils import load_module_from_name
from logilab.common.cache import Cache
from logilab.common.astng import ASTNGBuildingException
from logilab.common.modutils import modpath_from_file, get_modules


def normalize_module_name(mod_name):
    """ normalize a module name (i.e remove trailing __init__ if any)
    """
    parts = mod_name.split('.')
    if parts[-1] == '__init__':
        return '.'.join(parts[:-1])
    return mod_name

from logilab.common.configuration import OptionsProviderMixIn


class ASTNGManager(OptionsProviderMixIn):
    """build astng from files or modules
    """
    name = 'astng loader'
    options = (("ignore",
                {'action' :"append", 'type' : "string", 'metavar' : "<file>",
                 'dest' : "black_list", "default" : ('CVS',),
                 'help' : "Add <file> (may be a directory) to the black list\
. It should be a base name, not a path. You may set this option multiple times\
."}),
               ("project",
                {'default': "No Name", 'type' : 'string',
                 'metavar' : '<project name>',
                 'help' : 'set the project name.'}),
               )
    brain = {}    
    def __init__(self):
        """use the Borg pattern"""
        self.__dict__ = ASTNGManager.brain
        if not self.__dict__:
            OptionsProviderMixIn.__init__(self)
            from logilab.common.astng.builder import ASTNGBuilder
            self._builder = ASTNGBuilder()
            self._modules_by_name = Cache(200)

    def set_cache_size(self, cache_size):
        """set the cache size
        """
        self._modules_by_name.size = cache_size        

    def astng_from_class(self, klass, module=None):
        """get astng for the given class
        """
        if module is None:
            try:
                module = klass.__module__
            except AttributeError:
                raise ASTNGBuildingException(
                    'Unable to get module for class %s' % klass)
        module_node = self.astng_from_module_name(module)
        return module_node.resolve(klass.__name__, living=0)

    def astng_from_module(self, module, mod_name=None):
        """given an imported module, return the astng object
        """
        try:
            return self._modules_by_name[mod_name or module.__name__]
        except:
            if hasattr(module, '__file__'):
                # this is required to make works relative imports in
                # analyzed code
                sys.path.insert(0, dirname(module.__file__))
            try:
                astng = self._builder.build_from_module(module, mod_name)
                # update caches
                self._modules_by_name[astng.name] = astng
                return astng
            finally:
                # clean path
                if hasattr(module, '__file__'):
                    sys.path.pop(0)

    def astng_from_module_name(self, mod_name):
        """ given a module name, return the astng object
        """
        norm_mod_name = normalize_module_name(mod_name)
        try:
            return self._modules_by_name[norm_mod_name]
        except KeyError:
            try:
                module = load_module_from_name(norm_mod_name)
            except Exception, ex:
                msg = 'Unable to load module %s (%s)' % (mod_name, ex)
                raise ASTNGBuildingException(msg)
        # return the astng representation
        return self.astng_from_module(module, norm_mod_name)

    def project_from_files(self, files, func_wrapper,
                           project_name=None, black_list=None):
        """return a Project from a list of files or modules
        """
        # insert current working directory to the python path to have a correct
        # behaviour
        sys.path.insert(0, os.getcwd())
        try:
            # build the project representation
            project_name = project_name or self.config.project
            black_list = black_list or self.config.black_list
            project = Project(project_name)
            modnames = []
            for modname in files:
                if modname[-3:] == '.py' or modname.find(os.sep) > -1:
                    modname = '.'.join(modpath_from_file(modname))
                astng = func_wrapper(self.astng_from_module_name, modname)
                if astng is None:
                    continue
                project.base_file = project.base_file or astng.file
                project.add_module(astng)
                # recurse in package except if __init__ was explicitly given
                if not modname.endswith('.__init__') and astng.package:
                    # recurse on others packages / modules if this is a package
                    for submod in get_modules(modname, dirname(astng.file),
                                              black_list):
                        astng = func_wrapper(self.astng_from_module_name, submod)
                        if astng is None:
                            continue
                        project.add_module(astng)
            return project
        finally:
            sys.path.pop(0)
    
class Project:
    """a project handle a set of modules"""
    def __init__(self, name=''):
        self.name = name
        self.base_file = None
        self.modules = []
        self._modules = {}
        
    def add_module(self, node):
        self._modules[node.name] = node
        self.modules.append(node)
        
    def get_module(self, name):
        return self._modules[name]
    
    def getChildNodes(self):
        return self.modules

    def __repr__(self):
        return '<Project %r at %s (%s modules)>' % (self.name, id(self),
                                                    len(self.modules))
        
if __name__ == '__main__':
    obj_code = ASTNGManager().astng_from_module_name(sys.argv[1])
    print repr(obj_code)
    print obj_code.locals.keys()
    #print [str(c.__class__) for c in obj_code.children]
    print obj_code.doc


