# Copyright (c) 2003 Sylvain Thenault (thenault@nerim.net)
# Copyright (c) 2003 Logilab
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
"""astng utilities
"""

__author__ = "Sylvain Thenault"
__revision__ = "$Id: utils.py,v 1.3 2003/12/10 08:15:09 syt Exp $"

import inspect

class IgnoreChild(Exception):
    """exception that maybe raised by visit methods to avoid children traversal
    """
    
class ASTWalker:
    """a walker visiting a tree in preorder, calling on the handler:
    
    * visit_<class name> on entering a node, where class name is the class of
    the node in lower case
    
    * leave_<class name> on leaving a node, where class name is the class of
    the node in lower case
    """
    def __init__(self, handler):
        self.handler = handler
        self._cache = {}
        
    def walk(self, node):
        """walk on the tree from <node>, getting callbacks from handler
        """
        try:            
            self.visit(node)
        except IgnoreChild:
            pass
        else:
            for child_node in node.getChildNodes():
                self.walk(child_node)
        self.leave(node)

    def get_callbacks(self, node):
        """get callbacks from handler for the visited node
        """
        klass = node.__class__
        methods = self._cache.get(klass)
        if methods is None:
            handler = self.handler
            kid = klass.__name__.lower()
            e_method = getattr(handler, 'visit_%s' % kid,
                               getattr(handler, 'visit_default', None))
            l_method = getattr(handler, 'leave_%s' % kid, 
                               getattr(handler, 'leave_default', None))
            self._cache[klass] = (e_method, l_method)
        else:
            e_method, l_method = methods
        return e_method, l_method
    
    def visit(self, node):
        """walk on the tree from <node>, getting callbacks from handler
        """
        method = self.get_callbacks(node)[0]
        if method is not None:
            method(node)
            
    def leave(self, node):
        """walk on the tree from <node>, getting callbacks from handler
        """
        method = self.get_callbacks(node)[1]
        if method is not None:
            method(node)

def is_interface(klass):
    """return true if the given class may be considered as an interface
    """
    if hasattr(klass, 'object'):
        klass = klass.object
        if klass is None:
            return 0
    if klass.__name__.endswith('Interface'):
        return 1
    for base in klass.__bases__:
        if is_interface(base):
            return 1
    return 0

def is_exception(klass):
    """return true if the given class may be considered as an exception
    """
    if hasattr(klass, 'object'):
        klass = klass.object
        if klass is None:
            return 0
    if klass.__name__.endswith('Exception'):
        return 1
    for base in klass.__bases__:
        if is_exception(base):
            return 1
    return 0

def get_interfaces(klass, herited=1, manager=None):
    """return a list of interfaces implemented by the given klass node
    """
    try:
        if herited:
            implements = klass.object.__implements__
        else:
            implements = klass.object.__dict__['__implements__']
    except (KeyError, AttributeError), ex:
        return []
    
    if manager is None:
        from logilab.common.astng.manager import ASTNGManager
        manager = ASTNGManager()
    if not type(implements) in (type(()), type([])):
        implements = (implements,)
    result = []
    for iface in implements:
        # FIXME:error handling
        if not inspect.isclass(iface):
            continue
        result.append(manager.astng_from_class(iface))
    return result

def get_raises(node):
    """return a list of exception raised in node
    """
    from logilab.common.astng import Raise
    result = []
    for child in node.getChildNodes():
        if isinstance(child ,Raise):
            if child.expr1:
                result.append(child.expr1)
        else:
            result += get_raises(child)
    return result

def get_returns(klass, herited=1, manager=None):
    """return a list of nodes used in return statements
    """
    from logilab.common.astng import Return
    result = []
    for child in klass.getChildNodes():
        if isinstance(child , Return):
            result.append(child.value)
        else:
            result += get_returns(child)
    return result
